@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 #include <errno.h>
 
@@ -15,16 +16,17 @@ ppm_write(FILE *fp, struct ida_image *img)
 {
     fprintf(fp,"P6\n"
 	    "# written by ida " VERSION "\n"
-	    "# http://bytesex.org/ida/\n"
+	    "# https://www.kraxel.org/blog/linux/fbida/\n"
 	    "%d %d\n255\n",
             img->i.width,img->i.height);
-    fwrite(img->data, img->i.height, 3*img->i.width, fp);
+    fwrite(ida_image_scanline(img, 0),
+           img->i.height, 3*img->i.width, fp);
     return 0;
 }
 
 static struct ida_writer ppm_writer = {
     label:  "PPM",
-    ext:    { "ppm", NULL},
+    ext:    { "ppm", NULL },
     write:  ppm_write,
 };
 
